/* 
 * Simple rwhod (c) Dima Maloff, 1997 
 * $Id: srwhod.c,v 1.1 1997/07/17 02:52:59 mff Exp mff $
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/param.h>
#include <netdb.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "rwhod.h"

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 120
#endif

#define AL_INTERVAL (3 * 60)

time_t get_uptime (void);
void send_data ();

struct whod mywd;
struct servent *sp;
char myname[MAXHOSTNAMELEN];
int s;
char note[] = "srwhod is running\n";

int main (int argc, char *argv[])
{
  int on = 1;
  struct sockaddr_in sin;

  if ((sp = getservbyname ("who", "udp")) == NULL)
  {
    fprintf (stderr, "rwhod: udp/who: unknown service\n");
    exit (1);
  }

  if ((s = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
  {
    perror ("socket");
    exit (1);
  }
  if (setsockopt (s, SOL_SOCKET, SO_BROADCAST, &on, sizeof (on)) < 0)
  {
    perror ("setsockopt(SO_BROADCAST)");
    exit (1);
  }
  memset (&sin, 0, sizeof (sin));
  sin.sin_family = AF_INET;
  sin.sin_port = sp->s_port;
  if (bind (s, (struct sockaddr *) &sin, sizeof (sin)) < 0)
  {
    perror ("bind");
    exit (1);
  }

  printf ("%s", note);
  for (;;)
  {
    send_data ();
    sleep (AL_INTERVAL);
  }
}

void send_data ()
{
  int i;
  int cc;
  char *cp;
  struct sockaddr_in sin;	/* who to send to (broadcast) */

  memset (&mywd, 0, sizeof (mywd));
  mywd.wd_vers = WHODVERSION;
  mywd.wd_type = WHODTYPE_STATUS;
  mywd.wd_sendtime = htonl (time (0));

  /* 
   * Establish host name as returned by system.
   */
  if (gethostname (myname, sizeof (myname) - 1) < 0)
  {
    perror ("gethostname");
    exit (1);
  }
  if ((cp = strchr (myname, '.')) != NULL)
    *cp = '\0';
  strncpy (mywd.wd_hostname, myname, sizeof (mywd.wd_hostname) - 1);
  mywd.wd_hostname[sizeof (mywd.wd_hostname) - 1] = 0;

  /* 
   * How to get load averages under OS/2? 
   */
  for (i = 0; i < 3; i++)
    mywd.wd_loadav[i] = htonl ((u_long) 0);

  mywd.wd_boottime = htonl (time (0) - get_uptime ());

  /* 
   * Say no one's logged 
   */
  cc = (char *) mywd.wd_we - (char *) &mywd;

  sin.sin_family = AF_INET;
  sin.sin_port = sp->s_port;
  sin.sin_addr.s_addr = 0xffffffff;
  if (sendto (s, (char *) &mywd, cc, 0, (struct sockaddr *) &sin, sizeof (sin)) == -1)
  {
    perror ("sendto");
    exit (1);
  }
}
